if ('serviceWorker' in navigator) {
  navigator.serviceWorker.register('./sw.js', {scope: './'}).then(function() {
      // Registration was successful. Now, check to see whether the Service Worker is controlling the page.
      if (navigator.serviceWorker.controller) {
        // If .controller is set, then this page is being actively controlled by the Service Worker.
        // Show the interface for sending messages to the service worker.
        showCommands();
      } else {
        // If .controller isn't set, then prompt the user to reload the page so that the Service Worker can take
        // control. Until that happens, the Service Worker's message handler won't be used.
        document.querySelector('#status').textContent =
          'Please reload this page to allow the service worker to take control.';
      }
  }).catch(function(error) {
    // Something went wrong during registration. The service-worker.js file
    // might be unavailable or contain a syntax error.
    document.querySelector('#status').textContent = error;
  });
} else {
  // The current browser doesn't support Service Workers.
  var aElement = document.createElement('a');
  aElement.href = 'http://www.chromium.org/blink/serviceworker/service-worker-faq';
  aElement.textContent = 'Service Workers are not supported in the current browser.';
  document.querySelector('#status').appendChild(aElement);
}

jio = jIO.createJIO({
          type: "indexeddb",
          database: "cribjs"
        })

function setStatus(statusMessage) {
  document.querySelector('#status').textContent = statusMessage;
}

function showCommands() {
  document.querySelector('#add').addEventListener('click', function() {
    var url = document.querySelector('#url').value;
    sendMessage({
      command: 'add',
      url: url,
      information: new Blob([document.querySelector('#information').value], {
          type: document.querySelector('#mimetype').value
        })
    }).then(function() {
      // If the promise resolves, just display a success message.
      setStatus('Added to cache:  ' + url + ' at ' +Date());
    }).catch(setStatus); // If the promise rejects, then setStatus will be called with the error.
  });

  document.querySelector('#delete').addEventListener('click', function() {
    sendMessage({
      command: 'delete',
      url: document.querySelector('#url').value
    }).then(function() {
      // If the promise resolves, just display a success message.
      setStatus('Deleted from cache.');
    }).catch(setStatus); // If the promise rejects, then setStatus will be called with the error.
  });

  document.querySelector('#list-contents').addEventListener('click', function() {
    sendMessage({
      command: 'keys'
    }).then(function(data) {
      var contentsElement = document.querySelector('#contents');
      // Clear out the existing items from the list.
      while (contentsElement.firstChild) {
        contentsElement.removeChild(contentsElement.firstChild);
      }

      // Add each cached URL to the list, one by one.
      data.urls.forEach(function(url) {
        var liElement = document.createElement('li');
        liElement.textContent = url;
        contentsElement.appendChild(liElement);
      });
    }).catch(setStatus); // If the promise rejects, then setStatus will be called with the error.
  });

  document.querySelector('#get').addEventListener('click', function() {
    console.log("bar");
    getContent(document.querySelector('#url').value,
               function() {
                 document.querySelector('#mimetype').value = this.getResponseHeader("content-type");
                 document.querySelector('#information').value = this.responseText;
                 }
              );
  })

  document.querySelector('#save-contents').addEventListener('click', function() {
    console.log('asking for the keys');
    sendMessage({
      command: 'keys'
    }).then(function(data) {
      // Add each cached URL to the list, one by one.
      console.log("foo");
      return new RSVP.Queue()
        .push(function(){
          console.log("put");
          return jio.put("cribjs", {})
        })
        .push(function(){
         var promise_list = [], i, i_len, url;
          for (i = 0, i_len = data.urls.length; i < i_len; i += 1) {
            url = new String(data.urls[i]);
            console.log(url);
            promise_list.push(jIO.util.ajax({'url': url, dataType: "blob"}))
          };
          return RSVP.all(promise_list)
        })
        .push(function (response_list) {
           console.log(response_list);
           var promise_list = [], i, i_len, url, response;
           for (i = 0, i_len = response_list.length; i < i_len; i += 1) {
             response = response_list[i];
             promise_list.push(
               jio.putAttachment('cribjs', response.target.responseURL,      
                 response.target.response
               )
             );
           };
           return RSVP.all(promise_list)
        })
        .push(undefined, function (error) {
                console.log( error );
         });


    }).catch(setStatus); // If the promise rejects, then setStatus will be called with the error.
  });

  document.querySelector('#load-contents').addEventListener('click', function() {
    console.log('asking for the keys');
    var url_list = [];
      return new RSVP.Queue()
        .push(function(){
          console.log("put");
          return jio.allAttachments("cribjs")
        })
        .push(function(response){
         var promise_list = [], key;
          for (key in response) {
            if (response.hasOwnProperty(key)) {
              console.log(key);
              url_list.push(key)
              promise_list.push(jio.getAttachment("cribjs", key));
            }
          };
          return RSVP.all(promise_list)
        })
        .push(function (response_list) {
           var promise_list = [], i, i_len, url, index, response, location, location_len;
           location = document.location.origin;
           location_len = location.length
           console.log(url_list);
           console.log(response_list);
           for (i = 0, i_len = response_list.length; i < i_len; i += 1) {
             url = url_list[i]
             index = url.indexOf(location);
             if(index != -1)
               url = url.substr(index + location_len);
              sendMessage({
                command: 'add',
                url: 'crib' + url,
                information: response_list[i]
                  })
           }          
        })
        .push(undefined, function (error) {
                console.log( error );
         });
  });  

  document.querySelector('#commands').style.display = 'block';
}

function sendMessage(message) {
  // This wraps the message posting/response in a promise, which will resolve if the response doesn't
  // contain an error, and reject with the error if it does. If you'd prefer, it's possible to call
  // controller.postMessage() and set up the onmessage handler independently of a promise, but this is
  // a convenient wrapper.
  return new Promise(function(resolve, reject) {
    var messageChannel = new MessageChannel();
    messageChannel.port1.onmessage = function(event) {
      if (event.data.error) {
        reject(event.data.error);
      } else {
        resolve(event.data);
      }
    };

    // This sends the message data as well as transferring messageChannel.port2 to the service worker.
    // The service worker can then use the transferred port to reply via postMessage(), which
    // will in turn trigger the onmessage handler on messageChannel.port1.
    // See https://html.spec.whatwg.org/multipage/workers.html#dom-worker-postmessage
    navigator.serviceWorker.controller.postMessage(message, [messageChannel.port2]);
  });
}

function getContent(url, reqListener) {
  var oReq = new XMLHttpRequest();
  oReq.onload = reqListener;
  oReq.open("get", url, true);
  oReq.send();
}